from lino.projects.cosi.settings import *
SITE = Site(globals(),title=Site.verbose_name+" demo") 
#~ DEBUG=True
# the following line should always be commented out in a checked-in version
#~ DATABASES['default']['NAME'] = ':memory:'



